// This file is part of the Attempto Java Packages.
// Copyright 2008-2009, Attempto Group, University of Zurich (see http://attempto.ifi.uzh.ch).
//
// The Attempto Java Packages is free software: you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// The Attempto Java Packages is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE. See the GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License along with the Attempto
// Java Packages. If not, see http://www.gnu.org/licenses/.

package ch.uzh.ifi.attempto.echocomp;

import java.util.HashMap;

import nextapp.echo2.app.ApplicationInstance;
import nextapp.echo2.app.Column;
import nextapp.echo2.app.Component;
import nextapp.echo2.app.TaskQueueHandle;

/**
 * This abstract class can be used to create components that are initialized asynchronously in the background.
 * This is makes sense for components that require some time to create themselves (e.g. because of time
 * expensive calculations). The possibly time consuming creation of the component can be done in a synchronized
 * way so that at most one such creation process (per application instance) is running at a time.
 * 
 * @author Tobias Kuhn
 */
public abstract class DelayedComponent extends Column {
	
	private static HashMap<ApplicationInstance, TaskQueueHandle> taskQueues = new HashMap<ApplicationInstance, TaskQueueHandle>();
	
	/**
	 * Creates a new delayed component which shows the temporary component until the real component is
	 * ready.
	 * 
	 * @param tempComponent The temporary component.
	 * @param synchronize Defines whether the calculation should be performed in a synchronized way.
	 */
	public DelayedComponent(Component tempComponent, boolean synchronize) {
		if (tempComponent != null) {
			add(tempComponent);
		}
		
		final ApplicationInstance application = ApplicationInstance.getActive();
		TaskQueueHandle taskQueueTemp = taskQueues.get(application);
		if (taskQueueTemp == null) {
			taskQueueTemp = application.createTaskQueue();
			taskQueues.put(application, taskQueueTemp);
		}
		final TaskQueueHandle taskQueue = taskQueueTemp;
		
		if (synchronize) {
			
			Thread thread = new Thread() {
				public void run() {
					synchronized (application) {
						final Component c = initComponent();
						application.enqueueTask(
							taskQueue,
							new Runnable() {
								public void run() {
									DelayedComponent.this.removeAll();
									DelayedComponent.this.add(c);
									finalizeAction();
								}
							}
						);
						try {
							sleep(500);
						} catch (InterruptedException ex) {}
					}
				}
			};
			thread.setPriority(Thread.MIN_PRIORITY);
			thread.start();
			
		} else {
			
			Thread thread = new Thread() {
				public void run() {
					final Component c = initComponent();
					application.enqueueTask(
						taskQueue,
						new Runnable() {
							public void run() {
								DelayedComponent.this.removeAll();
								DelayedComponent.this.add(c);
								finalizeAction();
							}
						}
					);
				}
			};
			thread.start();
			
		}
		
	}
	
	
	/**
	 * Creates a new delayed component with the given temporary component. The calculation is
	 * not synchronized.
	 * 
	 * @param tempComponent The temporary component.
	 */
	public DelayedComponent(Component tempComponent) {
		this(tempComponent, false);
	}
	
	
	/**
	 * Creates a new delayed component with no temporary component.
	 * 
	 * @param synchronize Defines whether the calculation should be performed in a synchronized way.
	 */
	public DelayedComponent(boolean synchronize) {
		this(null, synchronize);
	}
	
	
	/**
	 * Creates a new delayed component with no temporary component. The calculation is
	 * not synchronized.
	 */
	public DelayedComponent() {
		this(null, false);
	}
	
	/**
	 * This method should contain the (possibly time-consuming) operations to create the actual GUI
	 * component. This operation will be performed asynchronously. As soon as it is finished, the
	 * temporary component (if present) is replaced by the component this method returns.
	 * 
	 * @return The GUI component.
	 */
	public abstract Component initComponent();
	
	/**
	 * Override this method to run code in the appliction context (e.g. for GUI changes) after the
	 * asynchronous operation has finished.
	 */
	public void finalizeAction() {
	}

}
