// This file is part of the Attempto Java Packages.
// Copyright 2008-2009, Attempto Group, University of Zurich (see http://attempto.ifi.uzh.ch).
//
// The Attempto Java Packages is free software: you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// The Attempto Java Packages is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE. See the GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License along with the Attempto
// Java Packages. If not, see http://www.gnu.org/licenses/.

package ch.uzh.ifi.attempto.preditor.text;

import ch.uzh.ifi.attempto.chartparser.Terminal;

/**
 * This is a general purpose implementation of a text element.
 * 
 * @author Tobias Kuhn
 */
public class BasicTextElement implements TextElement {
	
	private final String originalText;
	private String text;
	private Terminal category;
	private boolean capitalize;

	/**
	 * Creates a new basic text element.
	 * 
	 * @param text The text.
	 * @param category The category.
	 * @param capitalize true if the word should be capitalized in sentence-initial position.
	 */
	public BasicTextElement(String text, Terminal category, boolean capitalize) {
		this.text = text;
		this.originalText = text;
		this.category = category;
		this.capitalize = capitalize;
	}
	
	/**
	 * Creates a new basic text element.
	 * 
	 * @param text The text.
	 * @param categoryName The name of the category.
	 * @param capitalize true if the word should be capitalized in sentence-initial position.
	 */
	public BasicTextElement(String text, String categoryName, boolean capitalize) {
		this(text, new Terminal(categoryName), capitalize);
	}
	
	/**
	 * Creates a new basic text element. It is capitalized in sentence-initial position.
	 * 
	 * @param text The text.
	 * @param category The category.
	 */
	public BasicTextElement(String text, Terminal category) {
		this(text, category, true);
	}
	
	/**
	 * Creates a new basic text element. It is capitalized in sentence-initial position.
	 * 
	 * @param text The text.
	 * @param categoryName The name of the category.
	 */
	public BasicTextElement(String text, String categoryName) {
		this(text, new Terminal(categoryName), true);
	}
	
	/**
	 * Creates a new basic text element containing a number. The text element is capitalized in
	 * sentence-initial position.
	 * 
	 * @param number The number.
	 * @param category The category.
	 */
	public BasicTextElement(int number, Terminal category) {
		this(number + "", category, true);
	}
	
	/**
	 * Creates a new basic text element containing a number. The text element is capitalized in
	 * sentence-initial position.
	 * 
	 * @param number The number.
	 * @param categoryName The name of the category.
	 */
	public BasicTextElement(int number, String categoryName) {
		this(number + "", new Terminal(categoryName), true);
	}
	
	/**
	 * Creates a new basic text element where the text is the name of the category.
	 * 
	 * @param category The category.
	 * @param capitalize true if the word should be capitalized in sentence-initial position.
	 */
	public BasicTextElement(Terminal category, boolean capitalize) {
		this(category.getName(), category, capitalize);
	}
	
	/**
	 * Creates a new basic text element where the text is the name of the category.
	 * 
	 * @param categoryName The name of the category.
	 * @param capitalize true if the word should be capitalized in sentence-initial position.
	 */
	public BasicTextElement(String categoryName, boolean capitalize) {
		this(categoryName, new Terminal(categoryName), capitalize);
	}
	
	/**
	 * Creates a new basic text element where the text is the name of the category.
	 * The text element is capitalized in sentence-initial position.
	 * 
	 * @param category The category.
	 */
	public BasicTextElement(Terminal category) {
		this(category.getName(), category, true);
	}
	
	/**
	 * Creates a new basic text element where the text is the name of the category.
	 * The text element is capitalized in sentence-initial position.
	 * 
	 * @param categoryName The name of the category.
	 */
	public BasicTextElement(String categoryName) {
		this(categoryName, new Terminal(categoryName), true);
	}
	
	/**
	 * Creates a new basic text element containing a number which is also the name of the category.
	 * The text element is capitalized in sentence-initial position.
	 * 
	 * @param number The number.
	 */
	public BasicTextElement(int number) {
		this(number + "", new Terminal(number + ""), true);
	}
	
	public Terminal getCategory() {
		return category;
	}
	
	public String getText() {
		return text;
	}
	
	public void checkNeighborTextElements(TextElement precedingTextElement, TextElement nextTextElement) {
		if ((precedingTextElement == null || precedingTextElement.getText().matches("(\\.|\\?|\\!)")) && capitalize) {
			String f = originalText.substring(0, 1);
			text = f.toUpperCase() + originalText.substring(1);
		} else {
			text = originalText;
		}
		
		if (nextTextElement != null && text.matches("(A|a)n?")) {
			String nt = nextTextElement.getText().toLowerCase();
			boolean an = false;
			if (nt.matches("[aeiou].*")) an = true;
			if (nt.matches("[fhlmnrsx]")) an = true;
			if (nt.matches("[fhlmnrsx]-.*")) an = true;
			if (nt.equals("u")) an = false;
			if (nt.matches("u-.*")) an = false;
			if (nt.matches("u[rtn]i.*")) an = false;
			if (nt.matches("use.*")) an = false;
			if (nt.matches("uk.*")) an = false;
			
			if (an) {
				text = text.substring(0, 1) + "n";
			} else {
				text = text.substring(0, 1);
			}
		}
	}
	
	public boolean equals(Object obj) {
		if (obj instanceof BasicTextElement) {
			BasicTextElement other = (BasicTextElement) obj;
			return (this.text.equals(other.text) && this.category.equals(other.category));
		}
		return false;
	}

}
